@@ -79,6 +79,11 @@ test_that("handle_design_parameter throws error for degenerate design matrix", {
 
 })
 
+test_that("handle_design_parameter throws error for design matrix with NA's", {
+  data <- matrix(0, ncol = 20, nrow = 1)
+  df <- data.frame(var=c(1:19, NA))
+  expect_error(handle_design_parameter(~ var, data, col_data = df, reference_level = NULL))
+})
 
 test_that("glm_gp can handle no-row input", {
 
@@ -149,18 +154,21 @@ test_that("glm_gp can handle on_disk parameter", {
   fit_in_memory <- glm_gp(Y, design = ~ fav_food + city + age, col_data = data, on_disk = FALSE)
   fit_on_disk <- glm_gp(Y, design = ~ fav_food + city + age, col_data = data, on_disk = TRUE)
 
-  expect_equal(fit_in_memory[!names(fit_in_memory) %in% c("Mu", "data")], fit_on_disk[!names(fit_on_disk) %in% c("Mu", "data")])
+  expect_equal(fit_in_memory[!names(fit_in_memory) %in% c("Mu", "data", "Offset")], fit_on_disk[!names(fit_on_disk) %in% c("Mu", "data", "Offset")])
   expect_equal(c(fit_in_memory$Mu), c(fit_on_disk$Mu))
   expect_s4_class(fit_on_disk$Mu, "DelayedArray")
   expect_equal(c(assay(fit_in_memory$data)), c(assay(fit_on_disk$data)))
   expect_equal(class(fit_in_memory$data), class(fit_on_disk$data))
   expect_s4_class(assay(fit_on_disk$data), "DelayedArray")
+  expect_equal(c(fit_in_memory$Offset), c(fit_on_disk$Offset))
+  expect_s4_class(fit_on_disk$Offset, "DelayedArray")
 
 
   fit_on_disk2 <- glm_gp(Y_hdf5, design = ~ fav_food + city + age, col_data = data)
-  expect_equal(fit_on_disk[!names(fit_in_memory) %in% c("Mu", "data")], fit_on_disk2[!names(fit_on_disk2) %in% c("Mu", "data")])
+  expect_equal(fit_on_disk[!names(fit_in_memory) %in% c("Mu", "data", "Offset")], fit_on_disk2[!names(fit_on_disk2) %in% c("Mu", "data", "Offset")])
   expect_s4_class(fit_on_disk2$Mu, "DelayedArray")
   expect_s4_class(assay(fit_on_disk2$data), "DelayedArray")
+  expect_s4_class(fit_on_disk2$Offset, "DelayedArray")
 
   fit_in_memory2 <- glm_gp(Y_hdf5, design = ~ fav_food + city + age, col_data = data, on_disk = FALSE)
   expect_equal(fit_in_memory, fit_in_memory2)
@@ -186,7 +194,37 @@ test_that("glm_gp can handle SummarizedExperiment correctly", {
   fit_se <- glm_gp(se, design = ~ fav_food + city + age)
 
   expect_equal(fit, fit_se)
+})
 
+
+test_that("glm_gp can handle sparse input correctly", {
+
+
+  data <- data.frame(fav_food = sample(c("apple", "banana", "cherry"), size = 50, replace = TRUE),
+                     city = sample(c("heidelberg", "paris", "new york"), size = 50, replace = TRUE),
+                     age = rnorm(n = 50, mean = 40, sd = 15))
+  Y <- matrix(rnbinom(n = 10 * 50, mu = 3, size = 1/3.1), nrow = 10, ncol = 50)
+  rownames(Y) <- paste0("gene_", seq_len(10))
+  colnames(Y) <- paste0("person_", seq_len(50))
+  Y_sp <- as(Y, "dgCMatrix")
+
+  fit <- glm_gp(Y, design = ~ fav_food + city + age, col_data = data)
+  expect_error({
+    glm_gp(Y_sp, design = ~ fav_food + city + age, col_data = data)
+  })
+  fit_sp1 <- glm_gp(Y_sp, design = ~ fav_food + city + age, col_data = data, on_disk = FALSE)
+  fit_sp2 <- glm_gp(Y_sp, design = ~ fav_food + city + age, col_data = data, on_disk = TRUE)
+
+  expect_equal(fit, fit_sp1)
+
+  expect_equal(fit[!names(fit) %in% c("Mu", "data", "Offset")], fit_sp2[!names(fit_sp2) %in% c("Mu", "data", "Offset")])
+  expect_equal(c(fit$Mu), c(fit_sp2$Mu))
+  expect_s4_class(fit_sp2$Mu, "DelayedArray")
+  expect_equal(c(assay(fit$data)), c(assay(fit_sp2$data)))
+  expect_equal(class(fit$data), class(fit_sp2$data))
+  expect_s4_class(assay(fit_sp2$data), "DelayedArray")
+  expect_equal(c(fit$Offset), c(fit_sp2$Offset))
+  expect_s4_class(fit_sp2$Offset, "DelayedArray")
 })
 
 test_that("glm_gp can handle intercept model", {
@@ -231,7 +269,8 @@ test_that("glm_gp can handle design parameter of type vector", {
   colnames(design_mat) <- c("a", "b", "c")
   fit_vec <- glm_gp(Y, design = sample_assignment)
   fit_mat <- glm_gp(Y, design = design_mat)
-  expect_equal(fit_vec, fit_mat)
+  # Ignore design_formula
+  expect_equal(fit_vec[-c(10, 11)], fit_mat[-10])
 
 
   sample_assignment <- sample(c("a", "b", "c"), size = 50, replace = TRUE)
@@ -275,14 +314,20 @@ test_that("glm_gp gives error for wrong input data ", {
   expect_error(glm_gp(data = sp_mat))
 })
 
-test_that("glm_gp warns about mismatching col_data rownames ", {
+test_that("glm_gp can handle mismatching assay vs model matrix names ", {
 
   coldata <- data.frame(condition = c(rep("A", 4), rep("B", 3)), stringsAsFactors = FALSE)
   rownames(coldata) <- paste0("sample_", sample(1:7))
   Y <- matrix(numeric(0), ncol = 7, nrow = 0)
   colnames(Y) <- paste0("sample_", 1:7)
-  fit <- glm_gp(Y, design = ~ condition, col_data = coldata, size_factors = FALSE)
-  expect_equal(c(fit$model_matrix), c(model.matrix(~ condition, coldata)[colnames(Y), ]))
+  # Fails because SummarizedExperiment now checks that data and col_data match
+  expect_error(glm_gp(Y, design = ~ condition, col_data = coldata, size_factors = FALSE))
+
+  # Works because this is only reordering the model matrix and doesn't have a model matrix
+  mm <- model.matrix(~ condition, coldata)
+  fit <- glm_gp(Y, design = mm)
+  expect_equal(colnames(fit$data), paste0("sample_", 1:7))
+  expect_equal(rownames(fit$model_matrix), paste0("sample_", 1:7))
 })
 
 
@@ -363,6 +408,99 @@ test_that("lte_n_equal_rows and get_row_groups works", {
 
 
 
+test_that("NA's produced by fitBeta_fisher_scoring don't cause problems", {
+
+  skip_if_not(isNamespaceLoaded("devtools"))
+
+  n_genes <- 100
+  n_cells <- 500
+  sf <- rchisq(n = n_cells, df = 5)
+  sf <- sf / mean(sf)
+  gene_means <- 10^runif(n = n_genes, min = -3, max = 3)
+
+  Mu <- gene_means %*% t(sf)
+  Y <- matrix(rnbinom(n = n_genes * n_cells, size = 1 / 0.1, mu = Mu), nrow = n_genes, ncol = n_cells)
+  rownames(Y) <- paste0("Gene_", seq_len(n_genes))
+  colnames(Y) <- paste("Cell_", seq_len(n_cells))
+  cont <- rnorm(n_cells)
+  cont2 <- rnorm(n_cells)
+
+  fit_orig <- glm_gp(Y, design = ~ cont + cont2, size_factors = sf)
+
+
+  # mockr::with_mock and mockery::stub don't properly work yet
+  testthat::with_mock(
+    # Simulated failure to converge
+    fitBeta_fisher_scoring =  function(Y, model_matrix, exp_offset_matrix, thetas, beta_matSEXP, ridge_penalty_nl, tolerance, max_rel_mu_change, max_iter) {
+      res <- .Call(`_glmGamPoi_fitBeta_fisher_scoring`, Y, model_matrix, exp_offset_matrix, thetas, beta_matSEXP, ridge_penalty_nl, tolerance, max_rel_mu_change, max_iter)
+      if(! any(res$iter == 0)){
+        res$beta_mat[c(5, 17), ] <- NA
+        res$iter[c(5, 17)] <- 1000
+        res$deviance[c(5, 17)] <- NaN
+      }
+      res
+    },
+    {
+      # optim recovers problematic result
+      # although they are not 100% identical
+      expect_silent(fit <- glm_gp(Y, design = ~ cont + cont2, size_factors = sf))
+      expect_equal(fit_orig$Beta[-c(5, 17),], fit$Beta[-c(5, 17),])
+      expect_equal(fit_orig$Beta[c(5, 17),], fit$Beta[c(5, 17),], tolerance = 1e-3)
+      expect_equal(fit_orig$Mu[-c(5, 17),], fit$Mu[-c(5, 17),])
+      expect_equal(fit_orig$Mu[c(5, 17),], fit$Mu[c(5, 17),], tolerance = 1e-3)
+      expect_equal(fit_orig$overdispersions[-c(5, 17)], fit$overdispersions[-c(5, 17)])
+      expect_equal(fit_orig$overdispersions[c(5, 17)], fit$overdispersions[c(5, 17)], tolerance = 0.01)
+      expect_equal(fit_orig$overdispersion_shrinkage_list$ql_disp_estimate[-c(5, 17)], fit$overdispersion_shrinkage_list$ql_disp_estimate[-c(5, 17)])
+      expect_equal(fit_orig$overdispersion_shrinkage_list$ql_disp_estimate[c(5, 17)], fit$overdispersion_shrinkage_list$ql_disp_estimate[c(5, 17)], tolerance = 0.01)
+
+      expect_equal(fit_orig[-which(names(fit_orig) %in% c("Beta", "overdispersions", "overdispersion_shrinkage_list", "Mu"))],
+                   fit[-which(names(fit_orig) %in% c("Beta", "overdispersions", "overdispersion_shrinkage_list", "Mu"))])
+  })
+
+
+
+  testthat::with_mock(
+    # Simulated failure to converge
+    fitBeta_fisher_scoring =  function(Y, model_matrix, exp_offset_matrix, thetas, beta_matSEXP, ridge_penalty_nl, tolerance, max_rel_mu_change, max_iter) {
+      res <- .Call(`_glmGamPoi_fitBeta_fisher_scoring`, Y, model_matrix, exp_offset_matrix, thetas, beta_matSEXP, ridge_penalty_nl, tolerance, max_rel_mu_change, max_iter)
+      if(! any(res$iter == 0)){
+        res$beta_mat[c(5, 17), ] <- NA
+        res$iter[c(5, 17)] <- 1000
+        res$deviance[c(5, 17)] <- NaN
+      }
+      res
+    }, # Make sure that optim doesn't recover the correct result
+      estimate_betas_fisher_scoring = {
+        ebfs <- estimate_betas_fisher_scoring
+        args <- formals(ebfs)
+        args[["try_recovering_convergence_problems"]] <- FALSE
+        formals(ebfs) <- args
+        ebfs
+      },
+    {
+      # The function isn't crashed by internal NA's
+      expect_warning(fit <- glm_gp(Y, design = ~ cont + cont2, size_factors = sf))
+      expect_equal(sum(is.na(fit$Mu)), 2 * n_cells)
+      expect_equal(unname(which(is.na(fit$overdispersions))), c(5, 17))
+      expect_equal(unname(which(is.na(fit$overdispersion_shrinkage_list$ql_disp_shrunken))), c(5, 17))
+      expect_equal(unname(which(is.na(fit$Beta[, "Intercept"]))), c(5, 17))
+      expect_equal(unname(which(is.na(fit$Beta[, "cont"]))), c(5, 17))
+
+      expect_equal(fit_orig$Beta[-c(5, 17),], fit$Beta[-c(5, 17),], tolerance = 1e-4)
+      expect_equal(fit_orig$Mu[-c(5, 17),], fit$Mu[-c(5, 17),], tolerance = 1e-4)
+      expect_equal(fit_orig$overdispersions[-c(5, 17)], fit$overdispersions[-c(5, 17)])
+      expect_equal(fit_orig$overdispersion_shrinkage_list$ql_disp_estimate[-c(5, 17)],
+                   fit$overdispersion_shrinkage_list$ql_disp_estimate[-c(5, 17)], tolerance = 1e-3)
+
+      expect_silent(res <- test_de(fit, reduced_design = ~ 1))
+      expect_true(is.na(res$pval[5]))
+      expect_false(is.na(res$pval[4]))
+
+      expect_silent(res <- test_de(fit, contrast = cont))
+      expect_true(is.na(res$pval[5]))
+      expect_false(is.na(res$pval[4]))
+    })
+})
 
 
 
